<?php
/***************************************************************************
 * 
 * Name: Country Block
 * URI: http://jcore.net
 * Description: Blocks users from specific countries.
 * Author: Zan Keglic *
 * Version: 1.0
 * Tags: country,block, gpl
 * 
 **************************************************************************/
 
modules::register(
	'CountryBlocks', 
	_('Country Block'),
	_('Blocks users from specific countries.'));

class countryBlocks extends modules {
	
		/////// Variables
		var $adminPath 			= 'admin/modules/countryblocks';
		
		var $globalAdminPath;
		var $countriesPath;
		var $htaccessPath;
	
	function __construct() {
		
				languages::load('countryblocks');

				$this->globalAdminPath  = SITE_URL.'admin/?path='.$this->adminPath.'';
				$this->htaccessPath     = SITE_PATH.'.htaccess';
				$this->countriesPath    = SITE_PATH.'lib/modules/countryblocks/country_files/';
	}
	
	function __destruct() {
		
				languages::unload('countryblocks');

	}
	
	function installSQL() {
		sql::run(
			"CREATE TABLE IF NOT EXISTS `{countryblocks}` (" .
			" `ID` smallint(5) unsigned NOT NULL auto_increment," .
			" `Country` varchar(255) NOT NULL default ''," .
			" `Blocked` mediumint(9) NOT NULL default '0'," .
			" PRIMARY KEY  (`ID`)," .
			" KEY `Blocked` (`Blocked`)" .
			" ) ENGINE=MyISAM;");
		
		if (!sql::count("SELECT COUNT(*) AS `Rows` FROM `{countryblocks}`"))
			sql::run(
				"INSERT INTO `{countryblocks}` (`Country`, `Blocked`)".
				"VALUES".
				"('Australia', 0),".
				"('Austria', 0),".
				"('China', 0),".
				"('Germany', 0),".			
				"('Italy', 0),".
				"('Slovenia', 0),".
				"('USA', 0)");
		
		if (sql::error())
			return false;
		
		return true;
	}
	
	function installFiles() {

		$css = 
			".countryblocks {\n" .
			"	margin-bottom: 10px;\n" .
			"}\n" .
			".admin-link.countryblocks-unblock {" .
			"	background-image: url(\"http://icons.jcore.net/32/emblem-generic.png\");" .
			"}" .
			".admin-section-item.as-modules-countryblocks a {\n" .
			"	background-image: url(\"http://icons.jcore.net/48/emblem-danger.png\");\n" .
			"}\n";
		
		return
			files::save(SITE_PATH.'template/modules/css/countryblocks.css', $css);
	}
	
	function uninstallSQL() {
		return sql::run(
			" DROP TABLE IF EXISTS `{countryblocks}`");
	}
	
	function uninstallFiles() {
		return
			files::delete(SITE_PATH.'template/modules/css/countryblocks.css');
	}
	
	//// Display number of blocked countries.
	function countAdminItems() {
		if (!parent::installed($this))
			return 0;
		
		$records = sql::fetch(sql::run(
			" SELECT COUNT(*) AS `Rows`" .
			" FROM `{countryblocks}`" .
			" WHERE `Blocked` = 1 LIMIT 1"));
		return $records['Rows'];
	}
	
	function writeHtaccess($country) {
		$file = $this->htaccessPath;				
		if (!$data = $this->readCountryFile($country)) {
			tooltip::display(
				sprintf(_("Country file for \"%s\" couldn't be found!"), $country),
				TOOLTIP_ERROR);
			return false;
		}
		
		if (!$fh = @fopen($file, 'a')) {
			tooltip::display(
				_("Htaccess file couldn't be opened!")." " .
				sprintf(__("Please make sure \"%s\" is writable by me or contact webmaster."),
					SITE_PATH.'.htaccess'),
				TOOLTIP_ERROR);
			return false;
		}
		
		fwrite($fh, $data);
		fclose($fh);
		return true;
	}
	

	function findCountryData($string, $start, $end){
				$string = " ".$string;
				$ini = strpos($string,$start);
				if ($ini == 0) return "";
				$ini += strlen($start);
				$len = strpos($string,$end,$ini) - $ini;
				return substr($string,$ini,$len);
	}
	
	function deleteHtaccess($country) {
				$file = $this->htaccessPath;				
			
	        	if (!$fo   = @fopen($file, 'r')) {
					tooltip::display(
						_("Htaccess file couldn't be opened!")." " .
						sprintf(__("Please make sure \"%s\" is writable by me or contact webmaster."),
							SITE_PATH.'.htaccess'),
						TOOLTIP_ERROR);
					return false;
	        	}
	        	
				$data = fread($fo, filesize($file));
				fclose($fo);
						
					// Find and delete country data!				
					$country_data     = $this->findCountryData($data, "# Block ".$country."", "# End Block ".$country."");
					
					
					// Data cleaning!
						// Delete data for this country!
					$clean_data		  = str_replace($country_data, " ", $data);
						// Delete first and latest block for country!
					$clean_data		  = str_replace("# Block ".$country." # End Block ".$country."", "", $clean_data);
					  	// Clean all empty lines!
					$clean_data		  = preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n", $clean_data);
					
					// Write .htacces back!			
					if (!$fh = @fopen($file, 'w')) {
						tooltip::display(
							_("Htaccess file couldn't be opened!")." " .
							sprintf(__("Please make sure \"%s\" is writable by me or contact webmaster."),
								SITE_PATH.'.htaccess'),
							TOOLTIP_ERROR);
						return false;
					}
		
					fwrite($fh, $clean_data);
					fclose($fh);
					return true;					
	
		
	}
	
	function readCountryFile($country) {
		$file = $this->countriesPath."".$country.".txt";	
			
	        	if (!$fo   = @fopen($file, 'r'))
	        		return false;
	        	
				$data = fread($fo, filesize($file));
				fclose($fo);
				
					return $data;	
	}
	
	function getCountries() {
			$rows = sql::run(
			" SELECT * FROM `{countryblocks}`" .
			" ORDER BY `ID` ASC");	
			
					return $rows;
		
	}

	function getStatus($ID) {
		   $status = sql::fetch(sql::run(
			" SELECT `Blocked` FROM `{countryblocks}`" .
			" WHERE ID='".$ID."'"));
		   
		   		   
			 if($status['Blocked'] == "0") {
				 $status = _("Unblocked");
			 } else {
				 $status = _("Blocked");	
				}
				
		   			return $status;
		
	}
	
	function blockCountry($ID) {
			$country = sql::fetch(sql::run("SELECT `Country` FROM `{countryblocks}`".
								"WHERE ID = ".$ID.""));
			if (!$this->writeHtaccess($country['Country']))
				return false;
			
			sql::run(
				" UPDATE `{countryblocks}` SET " .
				" `Blocked` = 1" .
				" WHERE ID = ".$ID." ");	
			
			
					return true;
		
	}

	function unblockCountry($ID) {
			$country = sql::fetch(sql::run("SELECT `Country` FROM `{countryblocks}`".
								"WHERE ID = ".$ID.""));
				if (!$this->deleteHtaccess($country['Country']))
					return false;
					
			sql::run(
				" UPDATE `{countryblocks}` SET " .
				" `Blocked` = 0" .
				" WHERE ID = ".$ID." ");	
			
					
					return true;		
		
	}
	
	function displayAdminDescription() {
		echo 
			"<p>" .
				_("Check countries that should have blocked access.") .
			"</p>";
	}

	function displayAdmin() {
		
			
			admin::displayTitle(_('Country Block'));
			$this->displayAdminDescription();
		


//////////////// BLOCKING AND UNBLOCKING
			
			/// Start variables
				$id      = null;
				$block   = null;
				$unblock = null;
		
		if (isset($_GET['block'])) {
			$id = $_GET['id'];

				if ($this->blockCountry($id))
					 tooltip::display(
					_("Country has been blocked successfully."),
					TOOLTIP_SUCCESS);

		} elseif (isset($_GET['unblock'])) {
			$id = $_GET['id'];

				if ($this->unblockCountry($id))
					 tooltip::display(
					_("Country has been unblocked successfully."),
					TOOLTIP_SUCCESS);
		} 
//////////////// COUNTRIES CONTENT
			
			
				///// Data
				$rows = $this->getCountries();
			   
		    echo"<div class='admin-content'>";
			
			
				// Countries table
			echo "<table class='list'>" .
				"<thead>" .
				"<tr>" .
					"<th>"._("Status")."</th>" . 
					"<th>"._("Country")."</th>";
					
					
					
			if ($this->userPermissionType == USER_PERMISSION_TYPE_WRITE)
			echo
					"<th>"._("Block")."</th>" .
					"<th>"._("Unblock")."</th>";

			echo "</tr>" .
				 "</thead>" .
				 "<tbody>";		
					$i = 0;		
			while($row = sql::fetch($rows)) {
			echo 
				"<tr>" .
					"<td>" .
					 $this->getStatus($row['ID']) .
					"</td>" .
					"<td class='auto-width'>" .
						"<a href='".url::uri('id, edit, delete') .
							"&amp;id=".$row['ID']."' " .
						" class='bold'>" .
						$row['Country'] .
						"</a>" .
					"</td>";
			if ($this->userPermissionType == USER_PERMISSION_TYPE_WRITE)
				echo
					"<td>" .
						"<a class='admin-link important' " .
							"title='"._("Block this country")."' " .
							"href='".$this->globalAdminPath."&amp;id=".$row['ID']."&amp;block=1'>" .
						"</a>" .
					"</td>" .
					"<td>" .
						"<a class='admin-link countryblocks-unblock' " .
							"title='"._("Unblock this country")."' " .
							"href='".$this->globalAdminPath."&amp;id=".$row['ID']."&amp;unblock=1'>" .

						"</a>" .
					"</td>";					
					
					echo
				"</tr>";
					}
			 
				 
			echo "</tbody>" .
				 "</table>" .
				 "<br />";	
				 		
					echo "</div>";	//admin-content

	}
	
}
?>